@@ -19,6 +19,7 @@
 #define QGSRECTANGLE_H
 
 #include "qgis_core.h"
+#include "qgis.h"
 #include <iosfwd>
 #include <QDomDocument>
 #include <QRectF>
@@ -226,7 +227,7 @@ class CORE_EXPORT QgsRectangle
     /**
      * Returns the center point of the rectangle.
      */
-    QgsPointXY center() const { return QgsPointXY( mXmin + width() / 2, mYmin + height() / 2 ); }
+    QgsPointXY center() const { return QgsPointXY( mXmax * 0.5 + mXmin * 0.5, mYmin * 0.5 + mYmax * 0.5 ); }
 
     /**
      * Scale the rectangle around its center point.
@@ -261,6 +262,12 @@ class CORE_EXPORT QgsRectangle
       mYmax = centerY + newHeight / 2.0;
     }
 
+    /**
+     * Scale the rectangle around its \a center point.
+     * \since QGIS 3.4
+     */
+    QgsRectangle scaled( double scaleFactor, const QgsPointXY *center = nullptr ) const;
+
     /**
      * Grows the rectangle in place by the specified amount.
      * \see buffered()
@@ -316,7 +323,7 @@ class CORE_EXPORT QgsRectangle
     }
 
     /**
-     * Returns true when rectangle intersects with other rectangle.
+     * Returns TRUE when rectangle intersects with other rectangle.
      */
     bool intersects( const QgsRectangle &rect ) const
     {
@@ -330,7 +337,7 @@ class CORE_EXPORT QgsRectangle
     }
 
     /**
-     * Returns true when rectangle contains other rectangle.
+     * Returns TRUE when rectangle contains other rectangle.
      */
     bool contains( const QgsRectangle &rect ) const
     {
@@ -338,7 +345,7 @@ class CORE_EXPORT QgsRectangle
     }
 
     /**
-     * Returns true when rectangle contains a point.
+     * Returns TRUE when rectangle contains a point.
      */
     bool contains( const QgsPointXY &p ) const
     {
@@ -388,6 +395,17 @@ class CORE_EXPORT QgsRectangle
       combineExtentWith( point.x(), point.y() );
     }
 
+    /**
+     * Returns the distance from \a point to the nearest point on the boundary of the rectangle.
+     * \since QGIS 3.14
+     */
+    double distance( const QgsPointXY &point ) const
+    {
+      const double dx = std::max( std::max( mXmin - point.x(), 0.0 ), point.x() - mXmax );
+      const double dy = std::max( std::max( mYmin - point.y(), 0.0 ), point.y() - mYmax );
+      return std::sqrt( dx * dx + dy * dy );
+    }
+
     /**
      * Returns a rectangle offset from this one in the direction of the reversed vector.
      * \since QGIS 3.0
@@ -413,7 +431,7 @@ class CORE_EXPORT QgsRectangle
     QgsRectangle &operator+=( QgsVector v );
 
     /**
-     * Returns true if the rectangle is empty.
+     * Returns TRUE if the rectangle is empty.
      * An empty rectangle may still be non-null if it contains valid information (e.g. bounding box of a point).
      */
     bool isEmpty() const
@@ -466,7 +484,7 @@ class CORE_EXPORT QgsRectangle
 
     /**
      * Comparison operator
-     * \returns True if rectangles are equal
+     * \returns TRUE if rectangles are equal
      */
     bool operator==( const QgsRectangle &r1 ) const
     {
@@ -478,7 +496,7 @@ class CORE_EXPORT QgsRectangle
 
     /**
      * Comparison operator
-     * \returns False if rectangles are equal
+     * \returns FALSE if rectangles are equal
      */
     bool operator!=( const QgsRectangle &r1 ) const
     {
@@ -503,8 +521,8 @@ class CORE_EXPORT QgsRectangle
     }
 
     /**
-     * Returns true if the rectangle has finite boundaries. Will
-     * return false if any of the rectangle boundaries are NaN or Inf.
+     * Returns TRUE if the rectangle has finite boundaries. Will
+     * return FALSE if any of the rectangle boundaries are NaN or Inf.
      */
     bool isFinite() const
     {
@@ -541,11 +559,19 @@ class CORE_EXPORT QgsRectangle
       return QVariant::fromValue( *this );
     }
 
+    /**
+     * Returns a copy of this rectangle that is snapped to a grid with
+     * the specified \a spacing between the grid lines.
+     *
+     * \since QGIS 3.4
+     */
+    QgsRectangle snappedToGrid( double spacing ) const;
+
 #ifdef SIP_RUN
     SIP_PYOBJECT __repr__();
     % MethodCode
     QString str = QStringLiteral( "<QgsRectangle: %1>" ).arg( sipCpp->asWktCoordinates() );
-    sipRes = PyUnicode_FromString( str.toUtf8().data() );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
     % End
 #endif
 
