@@ -599,8 +599,14 @@ class GUI_EXPORT QgsSvgMarkerSymbolLayerWidget : public QgsSymbolLayerWidget, pr
   protected:
 
     void populateList();
-    //update gui for svg file (insert new path, update activation of gui elements for svg params)
-    void setGuiForSvg( const QgsSvgMarkerSymbolLayer *layer );
+
+    /**
+     * Updates the GUI to reflect the SVG marker symbol \a layer.
+     * \param layer SVG marker symbol layer
+     * \param skipDefaultColors if TRUE, the default fill and outline colors of the SVG file will not overwrite
+     * the ones from the symbol layer
+     */
+    void setGuiForSvg( const QgsSvgMarkerSymbolLayer *layer, bool skipDefaultColors = false );
 
     QgsSvgMarkerSymbolLayer *mLayer = nullptr;
 
@@ -1003,11 +1009,22 @@ class GUI_EXPORT QgsFontMarkerSymbolLayerWidget : public QgsSymbolLayerWidget, p
     CharacterWidget *widgetChar = nullptr;
 
   private slots:
+
+    /**
+     * Sets the font \a style.
+     * \since QGIS 3.14
+     */
+    void setFontStyle( const QString &style );
+
     void setOffset();
     void mSizeUnitWidget_changed();
     void mOffsetUnitWidget_changed();
     void mStrokeWidthUnitWidget_changed();
     void mStrokeWidthSpinBox_valueChanged( double d );
+
+    void populateFontStyleComboBox();
+    void mFontStyleComboBox_currentIndexChanged( int index );
+
     void mHorizontalAnchorComboBox_currentIndexChanged( int index );
     void mVerticalAnchorComboBox_currentIndexChanged( int index );
     void penJoinStyleChanged();
@@ -1017,6 +1034,9 @@ class GUI_EXPORT QgsFontMarkerSymbolLayerWidget : public QgsSymbolLayerWidget, p
 
     std::shared_ptr< QgsMarkerSymbol > mAssistantPreviewSymbol;
 
+    QFont mRefFont;
+    QFontDatabase mFontDB;
+
 };
 
 //////////
@@ -1059,7 +1079,8 @@ class GUI_EXPORT QgsCentroidFillSymbolLayerWidget : public QgsSymbolLayerWidget,
   private slots:
     void mDrawInsideCheckBox_stateChanged( int state );
     void mDrawAllPartsCheckBox_stateChanged( int state );
-
+    void mClipPointsCheckBox_stateChanged( int state );
+    void mClipOnCurrentPartOnlyCheckBox_stateChanged( int state );
 };
 
 
