@@ -19,25 +19,53 @@ email                : lrssvtml (at) gmail (dot) com
 Some portions of code were taken from https://code.google.com/p/pydee/
 """
 
-from PyQt4.QtCore import Qt, QSettings, QByteArray, QCoreApplication, QFile, QSize
-from PyQt4.QtGui import QDialog, QMenu, QColor, QShortcut, QKeySequence, QFont, QFontMetrics, QStandardItemModel, QStandardItem, QApplication, QClipboard
-from PyQt4.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
+from qgis.PyQt.QtCore import Qt, QByteArray, QCoreApplication, QFile, QSize
+from qgis.PyQt.QtWidgets import QDialog, QMenu, QShortcut, QApplication
+from qgis.PyQt.QtGui import QColor, QKeySequence, QFont, QFontMetrics, QStandardItemModel, QStandardItem, QClipboard, \
+    QFontDatabase
+from qgis.PyQt.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
 
 import sys
 import os
 import code
 import codecs
 import re
+import traceback
 
-from qgis.core import QgsApplication
-from ui_console_history_dlg import Ui_HistoryDialogPythonConsole
+from qgis.core import QgsApplication, QgsSettings, Qgis
+from .ui_console_history_dlg import Ui_HistoryDialogPythonConsole
 
-_init_commands = ["from qgis.core import *", "import qgis.utils",
-                  "from qgis.utils import iface"]
-_historyFile = unicode(QgsApplication.qgisSettingsDirPath() + "console_history.txt")
+_init_commands = ["import sys", "import os", "import re", "import math", "from qgis.core import *",
+                  "from qgis.gui import *", "from qgis.analysis import *", "from qgis._3d import *",
+                  "import processing", "import qgis.utils",
+                  "from qgis.utils import iface", "from qgis.PyQt.QtCore import *", "from qgis.PyQt.QtGui import *",
+                  "from qgis.PyQt.QtWidgets import *",
+                  "from qgis.PyQt.QtNetwork import *", "from qgis.PyQt.QtXml import *"]
+_historyFile = os.path.join(QgsApplication.qgisSettingsDirPath(), "console_history.txt")
 
 
 class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
+    DEFAULT_COLOR = "#4d4d4c"
+    KEYWORD_COLOR = "#8959a8"
+    CLASS_COLOR = "#4271ae"
+    METHOD_COLOR = "#4271ae"
+    DECORATION_COLOR = "#3e999f"
+    NUMBER_COLOR = "#c82829"
+    COMMENT_COLOR = "#8e908c"
+    COMMENT_BLOCK_COLOR = "#8e908c"
+    BACKGROUND_COLOR = "#ffffff"
+    CURSOR_COLOR = "#636363"
+    CARET_LINE_COLOR = "#efefef"
+    SINGLE_QUOTE_COLOR = "#718c00"
+    DOUBLE_QUOTE_COLOR = "#718c00"
+    TRIPLE_SINGLE_QUOTE_COLOR = "#eab700"
+    TRIPLE_DOUBLE_QUOTE_COLOR = "#eab700"
+    MARGIN_BACKGROUND_COLOR = "#efefef"
+    MARGIN_FOREGROUND_COLOR = "#636363"
+    SELECTION_BACKGROUND_COLOR = "#d7d7d7"
+    SELECTION_FOREGROUND_COLOR = "#303030"
+    MATCHED_BRACE_BACKGROUND_COLOR = "#b7f907"
+    MATCHED_BRACE_FOREGROUND_COLOR = "#303030"
 
     def __init__(self, parent=None):
         super(ShellScintilla, self).__init__(parent)
@@ -48,16 +76,25 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.opening = ['(', '{', '[', "'", '"']
         self.closing = [')', '}', ']', "'", '"']
 
-        self.settings = QSettings()
+        self.settings = QgsSettings()
 
         # Enable non-ascii chars for editor
         self.setUtf8(True)
 
         self.new_input_line = True
 
+        # Set the default font
+        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
+        self.setFont(font)
+        self.setMarginsFont(font)
+        # Margin 0 is used for line numbers
         self.setMarginWidth(0, 0)
         self.setMarginWidth(1, 0)
         self.setMarginWidth(2, 0)
+        self.setMarginsFont(font)
+        self.setMarginWidth(1, "00000")
+        self.setMarginType(1, 5)
+        self.setCaretLineVisible(False)
 
         self.buffer = []
 
@@ -67,7 +104,8 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
             self.runsource(line)
 
         self.history = []
-        self.historyIndex = 0
+        self.softHistory = ['']
+        self.softHistoryIndex = 0
         # Read history command file
         self.readHistoryFile()
 
@@ -76,7 +114,6 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         # Brace matching: enable for a brace immediately before or after
         # the current position
         self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
-        self.setMatchedBraceBackgroundColor(QColor("#b7f907"))
 
         # Current line visible with special background color
         self.setCaretWidth(2)
@@ -89,12 +126,12 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.SendScintilla(QsciScintilla.SCI_SETHSCROLLBAR, 0)
 
         # not too small
-        #self.setMinimumSize(500, 300)
+        # self.setMinimumSize(500, 300)
 
         self.setWrapMode(QsciScintilla.WrapCharacter)
         self.SendScintilla(QsciScintilla.SCI_EMPTYUNDOBUFFER)
 
-        ## Disable command key
+        # Disable command key
         ctrl, shift = self.SCMOD_CTRL << 16, self.SCMOD_SHIFT << 16
         self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('L') + ctrl)
         self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('T') + ctrl)
@@ -103,7 +140,7 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('Y') + ctrl)
         self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('L') + ctrl + shift)
 
-        ## New QShortcut = ctrl+space/ctrl+alt+space for Autocomplete
+        # New QShortcut = ctrl+space/ctrl+alt+space for Autocomplete
         self.newShortcutCSS = QShortcut(QKeySequence(Qt.CTRL + Qt.SHIFT + Qt.Key_Space), self)
         self.newShortcutCAS = QShortcut(QKeySequence(Qt.CTRL + Qt.ALT + Qt.Key_Space), self)
         self.newShortcutCSS.setContext(Qt.WidgetShortcut)
@@ -112,9 +149,8 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.newShortcutCSS.activated.connect(self.showHistory)
 
     def _setMinimumHeight(self):
-        fnt = self.settings.value("pythonConsole/fontfamilytext", "Monospace")
-        fntSize = self.settings.value("pythonConsole/fontsize", 10, type=int)
-        fm = QFontMetrics(QFont(fnt, fntSize))
+        font = self.lexer.defaultFont(0)
+        fm = QFontMetrics(font)
 
         self.setMinimumHeight(fm.height() + 10)
 
@@ -135,8 +171,17 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         else:
             self.setAutoCompletionSource(self.AcsNone)
 
-        cursorColor = self.settings.value("pythonConsole/cursorColor", QColor(Qt.black))
+        cursorColor = self.settings.value("pythonConsole/cursorColor", QColor(self.CURSOR_COLOR))
         self.setCaretForegroundColor(cursorColor)
+        self.setSelectionForegroundColor(QColor(
+            self.settings.value("pythonConsole/selectionForegroundColor", QColor(self.SELECTION_FOREGROUND_COLOR))))
+        self.setSelectionBackgroundColor(QColor(
+            self.settings.value("pythonConsole/selectionBackgroundColor", QColor(self.SELECTION_BACKGROUND_COLOR))))
+        self.setMatchedBraceBackgroundColor(QColor(self.settings.value("pythonConsole/matchedBraceBackgroundColor",
+                                                                       QColor(self.MATCHED_BRACE_BACKGROUND_COLOR))))
+        self.setMatchedBraceForegroundColor(QColor(self.settings.value("pythonConsole/matchedBraceForegroundColor",
+                                                                       QColor(self.MATCHED_BRACE_FOREGROUND_COLOR))))
+        self.setMarginsBackgroundColor(QColor(self.settings.value("pythonConsole/paperBackgroundColor", QColor(self.BACKGROUND_COLOR))))
 
         # Sets minimum height for input area based of font metric
         self._setMinimumHeight()
@@ -158,78 +203,79 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
             elif radioButtonSource == 'fromDocAPI':
                 self.autoCompleteFromAll()
 
-    def commandConsole(self, command):
+    def commandConsole(self, commands):
         if not self.is_cursor_on_last_line():
             self.move_cursor_to_end()
-        line, pos = self.getCursorPosition()
-        selCmdLenght = len(self.text(line))
-        self.setSelection(line, 4, line, selCmdLenght)
-        self.removeSelectedText()
-        if command == "processing":
-            # import Processing class
-            self.append('import processing')
-        elif command == "qtCore":
-            # import QtCore class
-            self.append('from PyQt4.QtCore import *')
-        elif command == "qtGui":
-            # import QtGui class
-            self.append('from PyQt4.QtGui import *')
-        self.entered()
+        for cmd in commands:
+            self.setText(cmd)
+            self.entered()
         self.move_cursor_to_end()
         self.setFocus()
 
     def setLexers(self):
         self.lexer = QsciLexerPython()
 
-        loadFont = self.settings.value("pythonConsole/fontfamilytext", "Monospace")
-        fontSize = self.settings.value("pythonConsole/fontsize", 10, type=int)
+        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
 
-        font = QFont(loadFont)
-        font.setFixedPitch(True)
-        font.setPointSize(fontSize)
-        font.setStyleHint(QFont.TypeWriter)
-        font.setStretch(QFont.SemiCondensed)
-        font.setLetterSpacing(QFont.PercentageSpacing, 87.0)
-        font.setBold(False)
+        loadFont = self.settings.value("pythonConsole/fontfamilytext")
+        if loadFont:
+            font.setFamily(loadFont)
+        fontSize = self.settings.value("pythonConsole/fontsize", type=int)
+        if fontSize:
+            font.setPointSize(fontSize)
 
         self.lexer.setDefaultFont(font)
-        self.lexer.setDefaultColor(QColor(self.settings.value("pythonConsole/defaultFontColor", QColor(Qt.black))))
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/commentFontColor", QColor(Qt.gray))), 1)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/keywordFontColor", QColor(Qt.darkGreen))), 5)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/classFontColor", QColor(Qt.blue))), 8)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/methodFontColor", QColor(Qt.darkGray))), 9)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/decorFontColor", QColor(Qt.darkBlue))), 15)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/commentBlockFontColor", QColor(Qt.gray))), 12)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/singleQuoteFontColor", QColor(Qt.blue))), 4)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/doubleQuoteFontColor", QColor(Qt.blue))), 3)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/tripleSingleQuoteFontColor", QColor(Qt.blue))), 6)
-        self.lexer.setColor(QColor(self.settings.value("pythonConsole/tripleDoubleQuoteFontColor", QColor(Qt.blue))), 7)
+        self.lexer.setDefaultColor(
+            QColor(self.settings.value("pythonConsole/defaultFontColor", QColor(self.DEFAULT_COLOR))))
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/commentFontColor", QColor(self.COMMENT_COLOR))),
+                            1)
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/numberFontColor", QColor(self.NUMBER_COLOR))), 2)
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/keywordFontColor", QColor(self.KEYWORD_COLOR))),
+                            5)
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/classFontColor", QColor(self.CLASS_COLOR))), 8)
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/methodFontColor", QColor(self.METHOD_COLOR))), 9)
+        self.lexer.setColor(QColor(self.settings.value("pythonConsole/decorFontColor", QColor(self.DECORATION_COLOR))),
+                            15)
+        self.lexer.setColor(
+            QColor(self.settings.value("pythonConsole/commentBlockFontColor", QColor(self.COMMENT_BLOCK_COLOR))), 12)
+        self.lexer.setColor(
+            QColor(self.settings.value("pythonConsole/singleQuoteFontColor", QColor(self.SINGLE_QUOTE_COLOR))), 4)
+        self.lexer.setColor(
+            QColor(self.settings.value("pythonConsole/doubleQuoteFontColor", QColor(self.DOUBLE_QUOTE_COLOR))), 3)
+        self.lexer.setColor(QColor(
+            self.settings.value("pythonConsole/tripleSingleQuoteFontColor", QColor(self.TRIPLE_SINGLE_QUOTE_COLOR))), 6)
+        self.lexer.setColor(QColor(
+            self.settings.value("pythonConsole/tripleDoubleQuoteFontColor", QColor(self.TRIPLE_DOUBLE_QUOTE_COLOR))), 7)
+        self.lexer.setColor(
+            QColor(self.settings.value("pythonConsole/defaultFontColorEditor", QColor(self.DEFAULT_COLOR))), 13)
         self.lexer.setFont(font, 1)
         self.lexer.setFont(font, 3)
         self.lexer.setFont(font, 4)
+        self.lexer.setFont(font, QsciLexerPython.UnclosedString)
 
         for style in range(0, 33):
-            paperColor = QColor(self.settings.value("pythonConsole/paperBackgroundColor", QColor(Qt.white)))
+            paperColor = QColor(
+                self.settings.value("pythonConsole/paperBackgroundColor", QColor(self.BACKGROUND_COLOR)))
             self.lexer.setPaper(paperColor, style)
 
         self.api = QsciAPIs(self.lexer)
-        chekBoxAPI = self.settings.value("pythonConsole/preloadAPI", True, type=bool)
-        chekBoxPreparedAPI = self.settings.value("pythonConsole/usePreparedAPIFile", False, type=bool)
-        if chekBoxAPI:
+        checkBoxAPI = self.settings.value("pythonConsole/preloadAPI", True, type=bool)
+        checkBoxPreparedAPI = self.settings.value("pythonConsole/usePreparedAPIFile", False, type=bool)
+        if checkBoxAPI:
             pap = os.path.join(QgsApplication.pkgDataPath(), "python", "qsci_apis", "pyqgis.pap")
             self.api.loadPrepared(pap)
-        elif chekBoxPreparedAPI:
+        elif checkBoxPreparedAPI:
             self.api.loadPrepared(self.settings.value("pythonConsole/preparedAPIFile"))
         else:
             apiPath = self.settings.value("pythonConsole/userAPI", [])
             for i in range(0, len(apiPath)):
-                self.api.load(unicode(apiPath[i]))
+                self.api.load(apiPath[i])
             self.api.prepare()
             self.lexer.setAPIs(self.api)
 
         self.setLexer(self.lexer)
 
-    ## TODO: show completion list for file and directory
+    # TODO: show completion list for file and directory
 
     def getText(self):
         """ Get the text as a unicode string. """
@@ -252,30 +298,38 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
     def get_end_pos(self):
         """Return (line, index) position of the last character"""
         line = self.lines() - 1
-        return (line, len(self.text(line)))
+        return line, len(self.text(line))
+
+    def is_cursor_at_start(self):
+        """Return True if cursor is at the end of text"""
+        cline, cindex = self.getCursorPosition()
+        return cline == 0 and cindex == 0
 
     def is_cursor_at_end(self):
         """Return True if cursor is at the end of text"""
         cline, cindex = self.getCursorPosition()
         return (cline, cindex) == self.get_end_pos()
 
+    def move_cursor_to_start(self):
+        """Move cursor to start of text"""
+        self.setCursorPosition(0, 0)
+        self.ensureCursorVisible()
+        self.ensureLineVisible(0)
+        self.displayPrompt(False)
+
     def move_cursor_to_end(self):
         """Move cursor to end of text"""
         line, index = self.get_end_pos()
         self.setCursorPosition(line, index)
         self.ensureCursorVisible()
         self.ensureLineVisible(line)
+        self.displayPrompt(False)
 
     def is_cursor_on_last_line(self):
         """Return True if cursor is on the last line"""
         cline, _ = self.getCursorPosition()
         return cline == self.lines() - 1
 
-    def is_cursor_on_edition_zone(self):
-        """ Return True if the cursor is in the edition zone """
-        cline, cindex = self.getCursorPosition()
-        return cline == self.lines() - 1 and cindex >= 4
-
     def new_prompt(self, prompt):
         """
         Print a new prompt and save its (line, index) position
@@ -287,18 +341,26 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.ensureLineVisible(line)
 
     def displayPrompt(self, more=False):
-        self.append("... ") if more else self.append(">>> ")
-        self.move_cursor_to_end()
+        self.SendScintilla(QsciScintilla.SCI_MARGINSETTEXT, 0, str.encode("..." if more else ">>>"))
 
-    def updateHistory(self, command):
+    def syncSoftHistory(self):
+        self.softHistory = self.history[:]
+        self.softHistory.append('')
+        self.softHistoryIndex = len(self.softHistory) - 1
+
+    def updateSoftHistory(self):
+        self.softHistory[self.softHistoryIndex] = self.text()
+
+    def updateHistory(self, command, skipSoftHistory=False):
         if isinstance(command, list):
             for line in command:
                 self.history.append(line)
         elif not command == "":
             if len(self.history) <= 0 or \
-               command != self.history[-1]:
+                    command != self.history[-1]:
                 self.history.append(command)
-        self.historyIndex = len(self.history)
+        if not skipSoftHistory:
+            self.syncSoftHistory()
 
     def writeHistoryFile(self, fromCloseConsole=False):
         ok = False
@@ -318,17 +380,19 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
     def readHistoryFile(self):
         fileExist = QFile.exists(_historyFile)
         if fileExist:
-            rH = codecs.open(_historyFile, 'r', encoding='utf-8')
-            for line in rH:
-                if line != "\n":
-                    l = line.rstrip('\n')
-                    self.updateHistory(l)
+            with codecs.open(_historyFile, 'r', encoding='utf-8') as rH:
+                for line in rH:
+                    if line != "\n":
+                        l = line.rstrip('\n')
+                        self.updateHistory(l, True)
+            self.syncSoftHistory()
         else:
             return
 
     def clearHistory(self, clearSession=False):
         if clearSession:
             self.history = []
+            self.syncSoftHistory()
             msgText = QCoreApplication.translate('PythonConsole',
                                                  'Session and file history cleared successfully.')
             self.parent.callWidgetMessageBar(msgText)
@@ -349,44 +413,39 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         self.clearHistory(True)
 
     def showPrevious(self):
-        if self.historyIndex < len(self.history) and self.history:
-            line, pos = self.getCursorPosition()
-            selCmdLenght = len(self.text(line))
-            self.setSelection(line, 4, line, selCmdLenght)
-            self.removeSelectedText()
-            self.historyIndex += 1
-            if self.historyIndex == len(self.history):
-                self.insert("")
-                pass
-            else:
-                self.insert(self.history[self.historyIndex])
+        if self.softHistoryIndex < len(self.softHistory) - 1 and self.softHistory:
+            self.softHistoryIndex += 1
+            self.setText(self.softHistory[self.softHistoryIndex])
             self.move_cursor_to_end()
-            #self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
+            # self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
 
     def showNext(self):
-        if self.historyIndex > 0 and self.history:
-            line, pos = self.getCursorPosition()
-            selCmdLenght = len(self.text(line))
-            self.setSelection(line, 4, line, selCmdLenght)
-            self.removeSelectedText()
-            self.historyIndex -= 1
-            if self.historyIndex == len(self.history):
-                self.insert("")
-            else:
-                self.insert(self.history[self.historyIndex])
+        if self.softHistoryIndex > 0 and self.softHistory:
+            self.softHistoryIndex -= 1
+            self.setText(self.softHistory[self.softHistoryIndex])
             self.move_cursor_to_end()
-            #self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
+            # self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
 
     def keyPressEvent(self, e):
+        # update the live history
+        self.updateSoftHistory()
+
         startLine, startPos, endLine, endPos = self.getSelection()
 
         # handle invalid cursor position and multiline selections
-        if not self.is_cursor_on_edition_zone() or startLine < endLine:
-            # allow to copy and select
+        if startLine < endLine:
+            # allow copying and selecting
             if e.modifiers() & (Qt.ControlModifier | Qt.MetaModifier):
-                if e.key() in (Qt.Key_C, Qt.Key_A):
+                if e.key() == Qt.Key_C:
+                    # only catch and return from Ctrl-C here if there's a selection
+                    if self.hasSelectedText():
+                        QsciScintilla.keyPressEvent(self, e)
+                        return
+                elif e.key() == Qt.Key_A:
                     QsciScintilla.keyPressEvent(self, e)
-                return
+                    return
+                else:
+                    return
             # allow selection
             if e.modifiers() & Qt.ShiftModifier:
                 if e.key() in (Qt.Key_Left, Qt.Key_Right, Qt.Key_Home, Qt.Key_End):
@@ -395,50 +454,43 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
             # all other keystrokes get sent to the input line
             self.move_cursor_to_end()
 
+        if e.modifiers() & (
+                Qt.ControlModifier | Qt.MetaModifier) and e.key() == Qt.Key_C and not self.hasSelectedText():
+            # keyboard interrupt
+            sys.stdout.fire_keyboard_interrupt = True
+            return
+
         line, index = self.getCursorPosition()
         cmd = self.text(line)
+        hasSelectedText = self.hasSelectedText()
 
         if e.key() in (Qt.Key_Return, Qt.Key_Enter) and not self.isListActive():
             self.entered()
 
         elif e.key() in (Qt.Key_Left, Qt.Key_Home):
             QsciScintilla.keyPressEvent(self, e)
-            # check whether the cursor is moved out of the edition zone
-            newline, newindex = self.getCursorPosition()
-            if newline < line or newindex < 4:
-                # fix selection and the cursor position
-                if self.hasSelectedText():
-                    self.setSelection(line, self.getSelection()[3], line, 4)
-                else:
-                    self.setCursorPosition(line, 4)
 
         elif e.key() in (Qt.Key_Backspace, Qt.Key_Delete):
             QsciScintilla.keyPressEvent(self, e)
-            # check whether the cursor is moved out of the edition zone
-            _, newindex = self.getCursorPosition()
-            if newindex < 4:
-                # restore the prompt chars (if removed) and
-                # fix the cursor position
-                self.insert(cmd[:3 - newindex] + " ")
-                self.setCursorPosition(line, 4)
             self.recolor()
 
         elif (e.modifiers() & (Qt.ControlModifier | Qt.MetaModifier) and e.key() == Qt.Key_V) or \
-             (e.modifiers() & Qt.ShiftModifier and e.key() == Qt.Key_Insert):
+                (e.modifiers() & Qt.ShiftModifier and e.key() == Qt.Key_Insert):
             self.paste()
             e.accept()
 
         elif e.key() == Qt.Key_Down and not self.isListActive():
             self.showPrevious()
+
         elif e.key() == Qt.Key_Up and not self.isListActive():
             self.showNext()
-        ## TODO: press event for auto-completion file directory
+
+        # TODO: press event for auto-completion file directory
         else:
-            t = unicode(e.text())
+            t = e.text()
             self.autoCloseBracket = self.settings.value("pythonConsole/autoCloseBracket", False, type=bool)
             self.autoImport = self.settings.value("pythonConsole/autoInsertionImport", True, type=bool)
-            txt = cmd[:index].replace('>>> ', '').replace('... ', '')
-            ## Close bracket automatically
+            # Close bracket automatically
             if t in self.opening and self.autoCloseBracket:
                 i = self.opening.index(t)
                 if self.hasSelectedText() and startPos != 0:
@@ -447,28 +499,30 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
                     self.insert(self.opening[i] + selText + self.closing[i])
                     self.setCursorPosition(endLine, endPos + 2)
                     return
-                elif t == '(' and (re.match(r'^[ \t]*def \w+$', txt)
-                                   or re.match(r'^[ \t]*class \w+$', txt)):
-                        self.insert('):')
+                elif t == '(' and (re.match(r'^[ \t]*def \w+$', cmd)
+                                   or re.match(r'^[ \t]*class \w+$', cmd)):
+                    self.insert('):')
                 else:
                     self.insert(self.closing[i])
-            ## FIXES #8392 (automatically removes the redundant char
-            ## when autoclosing brackets option is enabled)
+            # FIXES #8392 (automatically removes the redundant char
+            # when autoclosing brackets option is enabled)
             elif t in [')', ']', '}'] and self.autoCloseBracket:
-                txt = self.text(line)
                 try:
-                    if txt[index - 1] in self.opening and t == txt[index]:
+                    if cmd[index - 1] in self.opening and t == cmd[index]:
                         self.setCursorPosition(line, index + 1)
                         self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
                 except IndexError:
                     pass
             elif t == ' ' and self.autoImport:
                 ptrn = r'^[ \t]*from [\w.]+$'
-                if re.match(ptrn, txt):
+                if re.match(ptrn, cmd):
                     self.insert(' import')
                     self.setCursorPosition(line, index + 7)
             QsciScintilla.keyPressEvent(self, e)
 
+        if len(self.text(0)) == 0 or hasSelectedText:
+            self.displayPrompt(False)
+
     def contextMenuEvent(self, e):
         menu = QMenu(self)
         subMenu = QMenu(menu)
@@ -477,11 +531,6 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         subMenu.addAction(
             QCoreApplication.translate("PythonConsole", "Show"),
             self.showHistory, 'Ctrl+Shift+SPACE')
-        subMenu.addSeparator()
-        subMenu.addAction(
-            QCoreApplication.translate("PythonConsole", "Save"),
-            self.writeHistoryFile)
-        subMenu.addSeparator()
         subMenu.addAction(
             QCoreApplication.translate("PythonConsole", "Clear File"),
             self.clearHistory)
@@ -511,7 +560,7 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         """
         self.setFocus()
         if e.button() == Qt.MidButton:
-            stringSel = unicode(QApplication.clipboard().text(QClipboard.Selection))
+            stringSel = QApplication.clipboard().text(QClipboard.Selection)
             if not self.is_cursor_on_last_line():
                 self.move_cursor_to_end()
             self.insertFromDropPaste(stringSel)
@@ -526,7 +575,7 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         XXX: It should reimplement the virtual QScintilla.paste method,
         but it seems not used by QScintilla code.
         """
-        stringPaste = unicode(QApplication.clipboard().text())
+        stringPaste = QApplication.clipboard().text()
         if self.is_cursor_on_last_line():
             if self.hasSelectedText():
                 self.removeSelectedText()
@@ -534,7 +583,7 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
             self.move_cursor_to_end()
         self.insertFromDropPaste(stringPaste)
 
-    ## Drag and drop
+    # Drag and drop
     def dropEvent(self, e):
         if e.mimeData().hasText():
             stringDrag = e.mimeData().text()
@@ -543,63 +592,66 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
             e.setDropAction(Qt.CopyAction)
             e.accept()
         else:
-            QsciScintillaCompat.dropEvent(self, e)
+            QsciScintilla.dropEvent(self, e)
 
     def insertFromDropPaste(self, textDP):
-        pasteList = unicode(textDP).splitlines()
+        pasteList = textDP.splitlines()
         if pasteList:
             for line in pasteList[:-1]:
                 cleanLine = line.replace(">>> ", "").replace("... ", "")
-                self.insert(unicode(cleanLine))
+                self.insert(cleanLine)
                 self.move_cursor_to_end()
-                self.runCommand(unicode(self.currentCommand()))
+                self.runCommand(self.currentCommand())
             if pasteList[-1] != "":
                 line = pasteList[-1]
                 cleanLine = line.replace(">>> ", "").replace("... ", "")
-                self.insert(unicode(cleanLine))
-                self.move_cursor_to_end()
+                curpos = self.getCursorPosition()
+                self.insert(cleanLine)
+                self.setCursorPosition(curpos[0], curpos[1] + len(cleanLine))
 
     def insertTextFromFile(self, listOpenFile):
         for line in listOpenFile[:-1]:
             self.append(line)
             self.move_cursor_to_end()
             self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
-            self.runCommand(unicode(self.currentCommand()))
-        self.append(unicode(listOpenFile[-1]))
+            self.runCommand(self.currentCommand())
+        self.append(listOpenFile[-1])
         self.move_cursor_to_end()
         self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
 
     def entered(self):
         self.move_cursor_to_end()
-        self.runCommand(unicode(self.currentCommand()))
+        self.runCommand(self.currentCommand())
         self.setFocus()
         self.move_cursor_to_end()
 
     def currentCommand(self):
-        linenr, index = self.getCursorPosition()
         string = self.text()
-        cmdLine = string[4:]
-        cmd = unicode(cmdLine)
+        cmd = string
         return cmd
 
     def runCommand(self, cmd):
         self.writeCMD(cmd)
         import webbrowser
         self.updateHistory(cmd)
-        if cmd in ('_pyqgis', '_api'):
+        version = 'master' if 'master' in Qgis.QGIS_VERSION.lower() else re.findall(r'^\d.[0-9]*', Qgis.QGIS_VERSION)[0]
+        if cmd in ('_pyqgis', '_api', '_cookbook'):
             if cmd == '_pyqgis':
-                webbrowser.open("http://qgis.org/pyqgis-cookbook/")
+                webbrowser.open("https://qgis.org/pyqgis/{}".format(version))
             elif cmd == '_api':
-                webbrowser.open("http://qgis.org/api/")
+                webbrowser.open("https://qgis.org/api/{}".format('' if version == 'master' else version))
+            elif cmd == '_cookbook':
+                webbrowser.open("https://docs.qgis.org/{}/en/docs/pyqgis_developer_cookbook/".format(
+                    'testing' if version == 'master' else version))
             more = False
         else:
             self.buffer.append(cmd)
-            src = u"\n".join(self.buffer)
-            more = self.runsource(src, "<input>")
+            src = "\n".join(self.buffer)
+            more = self.runsource(src)
             if not more:
                 self.buffer = []
-        ## prevents to commands with more lines to break the console
-        ## in the case they have a eol different from '\n'
+        # prevents to commands with more lines to break the console
+        # in the case they have a eol different from '\n'
         self.setText('')
         self.move_cursor_to_end()
         self.displayPrompt(more)
@@ -608,10 +660,25 @@ class ShellScintilla(QsciScintilla, code.InteractiveInterpreter):
         sys.stderr.write(txt)
 
     def writeCMD(self, txt):
+        if sys.stdout:
+            sys.stdout.fire_keyboard_interrupt = False
         if len(txt) > 0:
             getCmdString = self.text()
-            prompt = getCmdString[0:4]
-            sys.stdout.write(prompt + txt + '\n')
+            sys.stdout.write('>>> ' + txt + '\n')
+
+    def runsource(self, source, filename='<input>', symbol='single'):
+        if sys.stdout:
+            sys.stdout.fire_keyboard_interrupt = False
+        hook = sys.excepthook
+        try:
+            def excepthook(etype, value, tb):
+                self.write("".join(traceback.format_exception(etype, value, tb)))
+
+            sys.excepthook = excepthook
+
+            return super(ShellScintilla, self).runsource(source, filename, symbol)
+        finally:
+            sys.excepthook = hook
 
 
 class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
@@ -623,7 +690,7 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
         self.setWindowTitle(QCoreApplication.translate("PythonConsole",
                                                        "Python Console - Command History"))
         self.listView.setToolTip(QCoreApplication.translate("PythonConsole",
-                                                            "Double click on item to execute"))
+                                                            "Double-click on item to execute"))
         self.model = QStandardItemModel(self.listView)
 
         self._reloadHistory()
@@ -636,7 +703,7 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
 
     def _runHistory(self, item):
         cmd = item.data(Qt.DisplayRole)
-        self.parent.runCommand(unicode(cmd))
+        self.parent.runCommand(cmd)
 
     def _saveHistory(self):
         self.parent.writeHistoryFile(True)
@@ -656,8 +723,8 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
         itemsSelected = self.listView.selectionModel().selectedIndexes()
         if itemsSelected:
             item = itemsSelected[0].row()
-            ## Remove item from the command history (just for the current session)
+            # Remove item from the command history (just for the current session)
             self.parent.history.pop(item)
             self.parent.historyIndex -= 1
-            ## Remove row from the command history dialog
+            # Remove row from the command history dialog
             self.model.removeRow(item)
