@@ -165,7 +165,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
         self.cboInputLayer.setEditText(filename)
 
     def reloadInputLayer(self):
-        """ create the input layer and update available options """
+        """Creates the input layer and update available options """
         if self.mode != self.ASK_FOR_INPUT_MODE:
             return True
 
@@ -209,7 +209,7 @@ class DlgImportVector(QDialog, Ui_Dialog):
 
         srcCrs = self.inLayer.crs()
         if not srcCrs.isValid():
-            srcCrs = QgsCoordinateReferenceSystem(4326)
+            srcCrs = QgsCoordinateReferenceSystem("EPSG:4326")
         self.widgetSourceSrid.setCrs(srcCrs)
         self.widgetTargetSrid.setCrs(srcCrs)
 
